//
//  CameraPicker.swift
//  SampleApp
//
//  Created by Nishan Niraula on 15/2/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI
import UIKit

struct CameraPicker: UIViewControllerRepresentable {
    
    typealias UIViewControllerType = UIImagePickerController
    
    @Binding var image: UIImage?
    @Environment(\.presentationMode) var presentationMode
    
    func makeUIViewController(context: Context) -> UIImagePickerController {
        let picker = UIImagePickerController()
        picker.sourceType = .camera
        picker.delegate = context.coordinator
        return picker
    }
    
    func updateUIViewController(_ uiViewController: UIImagePickerController, context: Context) {
        // Left empty
    }
    
    func makeCoordinator() -> ImagePickerCoordinator {
        return ImagePickerCoordinator(picker: self)
    }
    
    // MARK:- Coordinator
    
    class ImagePickerCoordinator: NSObject, UIImagePickerControllerDelegate, UINavigationControllerDelegate {
        
        var imagePicker: CameraPicker
        
        init(picker: CameraPicker) {
            self.imagePicker = picker
        }
        
        func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
            
            if let selectedImage = info[.originalImage] as? UIImage {
                self.imagePicker.image = selectedImage
            }
            
            self.imagePicker.presentationMode.wrappedValue.dismiss()
        }
    }
}
