//
//  AppDelegate.swift
//  SampleAppLiveVideo
//
//  Created by Nutchaphon Rewik on 16/9/2563 BE.
//  Copyright © 2563 Amity. All rights reserved.
//

import UIKit
import AmityLiveVideoBroadcastKit

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        
        AppManager.shared.setup(completion: nil)
        
        // https://developer.apple.com/documentation/usernotifications/registering_your_app_with_apns
        UIApplication.shared.registerForRemoteNotifications()
        
        // Setup UNUserNotificationCenter to handle push notification.
        // https://developer.apple.com/documentation/usernotifications/
        UNUserNotificationCenter.current().delegate = self
        
        if let launchOptions = launchOptions,
           let userInfo = launchOptions[UIApplication.LaunchOptionsKey.remoteNotification] as? [AnyHashable: Any] {
            // User tap at push notification when the app is killed.
            if let streamId = getStreamId(fromNotificationPayload: userInfo) {
                // If the notification is coming from live video, we play the video.
                AppManager.shared.presentLiveVideo(forStreamId: streamId)
            }
        }
        
        return true
    }
    
    // MARK: UISceneSession Lifecycle

    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        return UISceneConfiguration(name: "Default Configuration", sessionRole: connectingSceneSession.role)
    }
    
    // MARK: Push Notification
    
    func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        // https://developer.apple.com/documentation/usernotifications/registering_your_app_with_apns
        // Forward Tokens to Your Provider Server
        AppManager.shared.sendDeviceTokenToServer(token: deviceToken)
    }
    
    func application(_ application: UIApplication, didFailToRegisterForRemoteNotificationsWithError error: Error) {
        print("Fail didFailToRegisterForRemoteNotificationsWithError: \(error.localizedDescription)")
    }

}

extension AppDelegate: UNUserNotificationCenterDelegate {
    
    func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        
        // After user touch at notification, iOS will trigger this function.
        
        // The payload in push notification contains essential data.
        // For example, streamId and title.
        // For more information please look at push notification payload document.
        
        // Here we examine the payload to react.
        let content = response.notification.request.content
        
        if let streamId = getStreamId(fromNotificationPayload: content.userInfo) {
            AppManager.shared.presentLiveVideo(forStreamId: streamId)
        }
        
        completionHandler()
        
    }
    
    func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        // For this sample app, we allow every push notification, to present while the app is in foreground.
        return completionHandler([.alert, .badge, .sound])
    }
    
    private func getStreamId(fromNotificationPayload payload: [AnyHashable: Any]) -> String? {
        
        guard
            
            let data = payload["data"] as? [String: Any],
            let streamings = data["videoStreamings"] as? Array<[String: Any]>,
            let streamInfo = streamings.first,
            let streamId = streamInfo["streamId"] as? String else {
            
            return nil
            
        }
        
        return streamId
        
    }
    
}
